//########################################################################
// This program is provided "as is" without any express 
// or implied warranty of any kind with respect to this software. 
// In particular the author shall not be liable for any direct, 
// indirect, special, incidental or consequential damages arising 
// in any way from use of the software.
//
//
// Author   : Jialong He, 
// Date     : July, 1999. 
//
// Contact  : Jialong_He@bigfoot.com, Jialong_He@homemail.com
// Web Page : www.bigfoot.com/~Jialong_He
//########################################################################
// DrawMag.cpp : implementation file
//

#include "stdafx.h"
#include "FIRDsgn.h"
#include "DrawMag.h"
#include <math.h>
#define FFTSize 1024

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDrawMag

CDrawMag::CDrawMag()
{

	m_BrushBlack.lbStyle = BS_SOLID;
	m_BrushBlack.lbColor = RGB(0, 0, 0);
	m_BrushBlack.lbHatch = 0;

	m_BrushCross.lbStyle = BS_HATCHED;
	m_BrushCross.lbColor = RGB(0, 255, 0);
	m_BrushCross.lbHatch = HS_CROSS;

	MagArr = NULL;
	MagLen = 0;              

}

CDrawMag::~CDrawMag()
{
	delete [] MagArr;
}


BEGIN_MESSAGE_MAP(CDrawMag, CWnd)
	//{{AFX_MSG_MAP(CDrawMag)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CDrawMag message handlers


void CDrawMag::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	CRect rect;
	
	dc.SetPolyFillMode(WINDING);

	CBrush brushBlack(m_BrushBlack.lbColor);
//	CBrush brushCross(HS_CROSS, RGB(0, 0, 0));
	GetClientRect(rect);

	dc.FillRect(&rect, &brushBlack);
	
//	dc.SelectObject(&brushCross);
//	dc.Rectangle(rect);

	//------------------------------
	// draw grid in green color
	//------------------------------
	int x, y, xstep, ystep;
	int H = rect.Height();
	int W = rect.Width();
	CPen GreenPen(PS_DOT, 1, RGB(0, 255, 0));
	dc.SelectObject(&GreenPen);
	
	xstep = W / 10;
	for (x=0; x<W; x+=xstep) 
	{
		dc.MoveTo(x, 0);
		dc.LineTo(x, H);
	}

	ystep = H / 10;
	for (y=0; y<H; y+=ystep) 
	{
		dc.MoveTo(0, y);
		dc.LineTo(W, y);
	}


	//-------------------------
	//	plot magnitude curve
	//-------------------------
	int Cnt;
	CPen RedPen(PS_SOLID, 2, RGB(255, 0, 0));
	dc.SelectObject(&RedPen);
		
	dc.MoveTo(0, ystep);
	for (Cnt=1; Cnt<MagLen; Cnt++) {
		x = int (float(Cnt) / MagLen * W); 
		y = int (10.0 - MagArr[Cnt] / 10.0  * ystep);
		dc.LineTo(x, y);
		dc.MoveTo(x, y);
	}

	// Do not call CWnd::OnPaint() for painting messages
}


typedef struct {
    double real, imag;
} COMPLEX;

//===========================================================
//  In-place radix 2 decimation in time FFT
//  The result will be placed in Src buffer. The Len must be
//  2**m, such as 1024, 4096.
//===========================================================
int fft(COMPLEX *x, int Len) {


    static COMPLEX *w=NULL;      /* used to store the w complex array */
    static int mstore = 0;       /* stores m for future reference */
    static int n = 1;            /* length of fft stored for future */
    COMPLEX u,temp,tm;
    COMPLEX *xi,*xip,*xj,*wptr;

    int m, i,j,k,l,le,windex;

    double arg,w_real,w_imag,wrecur_real,wrecur_imag,wtemp_real;


	//-------------------------------------------
	// Check if signal length is radix-2
	//-------------------------------------------
	m = int(log(Len)/log(2.0));
	int Radix2 = 1 << m;  // find nearest radix-2 length
	if (Radix2 != Len) {
		return(-1);
	}

    //-----------------------------------------------

    if(m != mstore) {

    /* free previously allocated storage and set new m */

	if(w != NULL) delete(w);
	mstore = m;
	if(m == 0) return (0);       /* if m=0 then done */

    /* n = 2**m = fft length */

        n = 1 << m;
        le = n/2;

    /* allocate the storage for w */

	w = new COMPLEX[le-1];
	if(w==NULL) {
		return(-1);
    }

	/* calculate the w values recursively */

        arg = 4.0*atan(1.0)/le;         /* PI/le calculation */
        wrecur_real = w_real = cos(arg);
        wrecur_imag = w_imag = -sin(arg);
        xj = w;
        for (j = 1 ; j < le ; j++) {
            xj->real = wrecur_real;
            xj->imag = wrecur_imag;
            xj++;
            wtemp_real = wrecur_real*w_real - wrecur_imag*w_imag;
            wrecur_imag = wrecur_real*w_imag + wrecur_imag*w_real;
            wrecur_real = wtemp_real;
        }
    }

	/* start fft */

    le = n;
    windex = 1;
    for (l = 0 ; l < m ; l++) {
        le = le/2;

	/* first iteration with no multiplies */

        for(i = 0 ; i < n ; i = i + 2*le) {
            xi = x + i;
            xip = xi + le;
            temp.real = xi->real + xip->real;
            temp.imag = xi->imag + xip->imag;
            xip->real = xi->real - xip->real;
            xip->imag = xi->imag - xip->imag;
            *xi = temp;
        }

	/* remaining iterations use stored w */

        wptr = w + windex - 1;
        for (j = 1 ; j < le ; j++) {
            u = *wptr;
            for (i = j ; i < n ; i = i + 2*le) {
                xi = x + i;
                xip = xi + le;
                temp.real = xi->real + xip->real;
                temp.imag = xi->imag + xip->imag;
                tm.real = xi->real - xip->real;
                tm.imag = xi->imag - xip->imag;             
                xip->real = tm.real*u.real - tm.imag*u.imag;
                xip->imag = tm.real*u.imag + tm.imag*u.real;
                *xi = temp;
            }
            wptr = wptr + windex;
        }
        windex = 2*windex;
    }            

	/* rearrange data by bit reversing */

    j = 0;
    for (i = 1 ; i < (n-1) ; i++) {
        k = n/2;
        while(k <= j) {
            j = j - k;
            k = k/2;
        }
        j = j + k;
        if (i < j) {
            xi = x + i;
            xj = x + j;
            temp = *xj;
            *xj = *xi;
            *xi = temp;
        }
    }

    return (0);

}

//====================================================
//  calculate impulse response of FIR filter
//====================================================
int CDrawMag::imp_response(double *FCoef, int FNum) {

	COMPLEX *Sig;
	int Cnt, RtCode;

	if (FNum > FFTSize ) {
		return(-1); // error
	}

	Sig    = new COMPLEX[FFTSize];
	if (Sig == NULL) {
		return(-1); // error
	}

	//---------------------------
	// copy coefficients to Sig
	//---------------------------
	for (Cnt=0; Cnt<FNum; Cnt++) {
		Sig[Cnt].real = FCoef[Cnt];
		Sig[Cnt].imag = 0.0;
	} 	

	//---------------------------
	// append zeros
	//---------------------------
	for (Cnt=FNum; Cnt<FFTSize; Cnt++) {
		Sig[Cnt].real = 0.0;
		Sig[Cnt].imag = 0.0;
	} 	

	//---------------------------
	// magnitude buffer 
	//---------------------------
	if (MagArr == NULL) {
	    MagLen = FFTSize/2;              
		MagArr = new double [MagLen];
	}

	//---------------------------
	// Do FFT
	//---------------------------
	RtCode = fft(Sig, FFTSize);
	if (RtCode <0 || MagArr == NULL) {
		delete [] Sig;
		return (-1);
	}

	//---------------------------
	// Magnitude in dB
	//---------------------------
	for (Cnt=0; Cnt<MagLen; Cnt++) {
		MagArr[Cnt] = 10.0 * log10(Sig[Cnt].real*Sig[Cnt].real + Sig[Cnt].imag*Sig[Cnt].imag);
	}
	MagArr[0] = 0.0;  // clear DC

	delete [] Sig;
	return(MagLen);
}
